//
//  ATKAppDelegate.m
//  ExampleApplication
//
//  Created by Nick Kitchener on 07/01/2013.
//  Copyright (c) 2013 ATIK. All rights reserved.
//

#import "ATKAppDelegate.h"
#import <ATIKOSXDrivers/Logger.h>

@implementation ATKAppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    Logger::globalLogger("ATIKOSXDrivers");
    Logger::setDebugLogging(false);
    
    [[NSUserDefaultsController sharedUserDefaultsController] setAppliesImmediately:YES]; // should be set already..
    
    // start up ATIK drivers
    _modernDrivers = [[ATIKOSXDrivers alloc]init];
    _legacyDrivers = [[ATIKOSXLegacyDrivers alloc]init];
    
    // deal with any existing FTDI mappings
    // If you have 16IC then use the following to add constant ids for each camera serial number
    id kvo = [[NSUserDefaultsController sharedUserDefaultsController] values];
    NSArray *legacyMappings = [kvo mutableArrayValueForKey:@"legacyMappings"];
    
    if(legacyMappings.count>0) {
        NSLog(@"Detected legacy mappings %@", legacyMappings);
        [legacyMappings enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            NSDictionary *objDictionary = obj;
            NSString *temp = [NSString stringWithFormat:@"0x%@", [objDictionary objectForKey:@"ftdiChipId"]];
            NSScanner *scanner=[NSScanner scannerWithString:temp];
            unsigned int chipId=0;
            [scanner scanHexInt:&chipId];
            NSString *serial = [objDictionary objectForKey:@"serialNumber"];
            if( (chipId==0)||(serial==nil)) {
                NSLog(@"Mapping is invalid - either/or zero chipid / null serial number");
            } else {
                NSLog(@"Adding legacy mapping 0x%08x for serial number %@", chipId, serial);
                
                [_legacyDrivers registerFTDIChipId:chipId forUSBSerialNumber:serial];
            };
        }];
    } else {
        NSLog(@"No Legacy mappings from preferences - adding Nick's 16IC Legacy mapping because he's lazy like that.");
        [_legacyDrivers registerFTDIChipId:0x0ee8f4c9 forUSBSerialNumber:@"A3001QH8"]; //0x0ee8f4c9 is my camera's id..
    }
    
    // We use the serviceManagement method to get the discovery/management interface
    id<Services> servicesModern = [_modernDrivers serviceManagement];
    id<Services> servicesLegacy = [_legacyDrivers serviceManagement];

    
    _controllerManager = [[ATKDeviceControllerManager alloc]init];
    [_controllerManager setModernDrivers:servicesModern];
    [_controllerManager setLegacyDrivers:servicesLegacy];
    
    // now we'll set the notification delegate - this object will receive
    // connection and disconnection events (this is optional but useful)
    [_modernDrivers setNotificationDelegate:self];
    [_legacyDrivers setNotificationDelegate:self];
    
    [_modernDrivers startSupport];
    [_legacyDrivers startSupport];
    
    _mainWindowController = [[ATKMainWindowController alloc] initWithWindowNibName:@"ATKMainWindowController"];
    [_mainWindowController setDeviceControllerManager:_controllerManager];
    
	[_mainWindowController showWindow:self];

    NSLog(@"Initialisation complete");
    
    // by this stage the drivers will have identited all the existing cameras connected
    // and will identify any connected cameras

    // next if we want to find the available services at any point:
    NSDictionary *availableModern = [servicesModern availableServices];
    NSLog(@"The following ATIK services are available: %@", availableModern);

    NSDictionary *availableLegacy = [servicesLegacy availableServices];
    NSLog(@"The following ATIK services are available: %@", availableLegacy);
    
}

-(void)applicationWillTerminate:(NSNotification *)notification {
    // release any claimed cameras
    
    // unhook the drivers
    [_modernDrivers endSupport];
    [_legacyDrivers endSupport];
    
    // release and let ARC release them when ready.
    _modernDrivers = nil;
    _legacyDrivers = nil;
}

-(IBAction)addTestCameraInstance:(id)sender {
    [_modernDrivers addTestProxyInstance:@"Test Camera" withType:@"Camera"];
}

-(IBAction)addTestLegacyCameraInstance:(id)sender {
    [_modernDrivers addTestProxyInstance:@"Test Legacy Camera" withType:@"Legacy"];
}

-(IBAction)addTestFilterWheelInstance:(id)sender {
    [_modernDrivers addTestProxyInstance:@"Test FilterWheel" withType:@"FilterWheel5"];
}

-(IBAction)removeTestCameraInstance:(id)sender {
    [_modernDrivers removeTestProxyInstance:@"Test Camera" withType:@"Camera"];
}

-(IBAction)removeTestFilterWheelInstance:(id)sender {
    [_modernDrivers removeTestProxyInstance:@"Test FilterWheel" withType:@"FilterWheel5"];
}

-(NSString*)driverVersion {
    return [ATIKOSXDrivers version];
}

@end


// note that thread(s) other than the main thread will enter here.
@implementation ATKAppDelegate (ATIKOSXDriverNotificationDelegate)

-(void)deviceConnected:(NSString*)identity {
    NSLog(@"The following ATIK device attached: %@", identity);
    
    // find the type of the connected device
    // Create a controller object
    // provide the newly attached device to the controller object
    // add the controller object to the main window
    
    [_controllerManager deviceConnected:identity];

}

-(void)deviceDisconnected:(NSString*)identity {
    NSLog(@"The following ATIK device detached: %@", identity);
    [_mainWindowController deactivateLiveViewHID];
    [_controllerManager deviceDisconnected:identity];
}


@end
